@@ -32,32 +32,12 @@
  * this file does not exist!!!!
  *
  */
-#define SPANDSP_NO_TIFF 1
-#include "spandsp.h"
 #include "switch_profile.h"
 
 #ifndef WIN32
 #include <switch_private.h>
 #endif
 
-/* for apr_pool_create and apr_pool_destroy */
-/* functions only used in this file so not exposed */
-#include <apr_pools.h>
-
-/* for apr_hash_make, apr_hash_pool_get, apr_hash_set */
-/* functions only used in this file so not exposed */
-#include <apr_hash.h>
-
-/* for apr_pvsprintf */
-/* function only used in this file so not exposed */
-#include <apr_strings.h>
-
-/* for apr_initialize and apr_terminate */
-/* function only used in this file so not exposed */
-#include <apr_general.h>
-
-#include <apr_portable.h>
-
 #ifdef HAVE_MLOCKALL
 #include <sys/mman.h>
 #endif
@@ -175,7 +155,7 @@ struct switch_core_session {
 	switch_log_level_t loglevel;
 	uint32_t soft_lock;
 	switch_ivr_dmachine_t *dmachine[2];
-	plc_state_t *plc;
+	switch_plc_state_t *plc;
 
 	switch_media_handle_t *media_handle;
 	uint32_t decoder_errors;
