@@ -843,6 +843,9 @@ typedef enum {
 	SWITCH_RTP_FLAG_PASSTHRU,
 	SWITCH_RTP_FLAG_SECURE_SEND_MKI,
 	SWITCH_RTP_FLAG_SECURE_RECV_MKI,
+	SWITCH_RTP_FLAG_SRTP_HANGUP_ON_ERROR,
+	SWITCH_RTP_FLAG_AUDIO_FIRE_SEND_RTCP_EVENT,
+	SWITCH_RTP_FLAG_VIDEO_FIRE_SEND_RTCP_EVENT,
 	SWITCH_RTP_FLAG_INVALID
 } switch_rtp_flag_t;
 
@@ -1135,6 +1138,7 @@ typedef enum {
 	SWITCH_MESSAGE_INDICATE_DEFLECT,
 	SWITCH_MESSAGE_INDICATE_VIDEO_REFRESH_REQ,
 	SWITCH_MESSAGE_INDICATE_DISPLAY,
+	SWITCH_MESSAGE_INDICATE_MEDIA_PARAMS,
 	SWITCH_MESSAGE_INDICATE_TRANSCODING_NECESSARY,
 	SWITCH_MESSAGE_INDICATE_AUDIO_SYNC,
 	SWITCH_MESSAGE_INDICATE_VIDEO_SYNC,
@@ -1171,6 +1175,7 @@ typedef enum {
 	SWITCH_MESSAGE_RESAMPLE_EVENT,
 	SWITCH_MESSAGE_HEARTBEAT_EVENT,
 	SWITCH_MESSAGE_INDICATE_SESSION_ID,
+	SWITCH_MESSAGE_INDICATE_PROMPT,
 	SWITCH_MESSAGE_INVALID
 } switch_core_session_message_types_t;
 
@@ -1324,13 +1329,17 @@ typedef enum {
 } switch_core_session_message_flag_enum_t;
 typedef uint32_t switch_core_session_message_flag_t;
 
+typedef struct switch_core_session switch_core_session_t;
+static inline switch_core_session_t *switch_core_session_type_check(switch_core_session_t *session) { return session; }
+static inline const char *switch_const_char_type_check(const char *str) { return str; }
+
 #define SWITCH_CHANNEL_LOG SWITCH_CHANNEL_ID_LOG, __FILE__, __SWITCH_FUNC__, __LINE__, NULL
 #define SWITCH_CHANNEL_LOG_CLEAN SWITCH_CHANNEL_ID_LOG_CLEAN, __FILE__, __SWITCH_FUNC__, __LINE__, NULL
 #define SWITCH_CHANNEL_SESSION_LOG_CLEAN(x) SWITCH_CHANNEL_ID_LOG_CLEAN, __FILE__, __SWITCH_FUNC__, __LINE__, switch_core_session_get_uuid((x))
 #define SWITCH_CHANNEL_EVENT SWITCH_CHANNEL_ID_EVENT, __FILE__, __SWITCH_FUNC__, __LINE__, NULL
-#define SWITCH_CHANNEL_SESSION_LOG(x) SWITCH_CHANNEL_ID_SESSION, __FILE__, __SWITCH_FUNC__, __LINE__, (const char*)(x)
+#define SWITCH_CHANNEL_SESSION_LOG(x) SWITCH_CHANNEL_ID_SESSION, __FILE__, __SWITCH_FUNC__, __LINE__, (const char*)switch_core_session_type_check(x)
 #define SWITCH_CHANNEL_CHANNEL_LOG(x) SWITCH_CHANNEL_ID_SESSION, __FILE__, __SWITCH_FUNC__, __LINE__, (const char*)switch_channel_get_session(x)
-#define SWITCH_CHANNEL_UUID_LOG(x) SWITCH_CHANNEL_ID_LOG, __FILE__, __SWITCH_FUNC__, __LINE__, (x)
+#define SWITCH_CHANNEL_UUID_LOG(x) SWITCH_CHANNEL_ID_LOG, __FILE__, __SWITCH_FUNC__, __LINE__, switch_const_char_type_check(x)
 
 typedef enum {
 	CCS_DOWN,
@@ -1450,6 +1459,7 @@ typedef enum {
 	CC_IO_OVERRIDE,
 	CC_RTP_RTT,
 	CC_MSRP,
+	CC_MUTE_VIA_MEDIA_STREAM,
 	/* WARNING: DO NOT ADD ANY FLAGS BELOW THIS LINE */
 	CC_FLAG_MAX
 } switch_channel_cap_t;
@@ -1529,6 +1539,7 @@ typedef enum {
 	CF_INTERCEPTED,
 	CF_VIDEO_REFRESH_REQ,
 	CF_MANUAL_VID_REFRESH,
+	CF_MANUAL_MEDIA_PARAMS,
 	CF_SERVICE_AUDIO,
 	CF_SERVICE_VIDEO,
 	CF_ZRTP_PASSTHRU_REQ,
@@ -1562,6 +1573,7 @@ typedef enum {
 	CF_T38_PASSTHRU,
 	CF_DROP_DTMF,
 	CF_REINVITE,
+	CF_NOSDP_REINVITE,
 	CF_AUTOFLUSH_DURING_BRIDGE,
 	CF_RTP_NOTIMER_DURING_BRIDGE,
 	CF_AVPF,
@@ -1589,7 +1601,7 @@ typedef enum {
 	CF_VIDEO_DECODED_READ,
 	CF_VIDEO_DEBUG_READ,
 	CF_VIDEO_DEBUG_WRITE,
-	CF_VIDEO_ONLY,
+	CF_NO_RECOVER,
 	CF_VIDEO_READY,
 	CF_VIDEO_MIRROR_INPUT,
 	CF_VIDEO_READ_FILE_ATTACHED,
@@ -1623,6 +1635,10 @@ typedef enum {
 	CF_STATE_REPEAT,
 	CF_WANT_DTLSv1_2,
 	CF_RFC7329_COMPAT,
+	CF_REATTACHED,
+	CF_VIDEO_READ_TAPPED,
+	CF_VIDEO_WRITE_TAPPED,
+	CF_DEVICES_CHANGED,
 	/* WARNING: DO NOT ADD ANY FLAGS BELOW THIS LINE */
 	/* IF YOU ADD NEW ONES CHECK IF THEY SHOULD PERSIST OR ZERO THEM IN switch_core_session.c switch_core_session_request_xml() */
 	CF_FLAG_MAX
@@ -1900,7 +1916,8 @@ typedef enum {
 	SMBF_SPY_VIDEO_STREAM_BLEG = (1 << 23),
 	SMBF_READ_VIDEO_PATCH = (1 << 24),
 	SMBF_READ_TEXT_STREAM = (1 << 25),
-	SMBF_FIRST = (1 << 26)
+	SMBF_FIRST = (1 << 26),
+	SMBF_PAUSE = (1 << 27)
 } switch_media_bug_flag_enum_t;
 typedef uint32_t switch_media_bug_flag_t;
 
@@ -2037,6 +2054,7 @@ typedef uint32_t switch_io_flag_t;
     SWITCH_EVENT_SEND_INFO
     SWITCH_EVENT_RECV_INFO
     SWITCH_EVENT_RECV_RTCP_MESSAGE
+    SWITCH_EVENT_SEND_RTCP_MESSAGE
     SWITCH_EVENT_CALL_SECURE
     SWITCH_EVENT_NAT            	- NAT Management (new/del/status)
     SWITCH_EVENT_RECORD_START
@@ -2132,6 +2150,7 @@ typedef enum {
 	SWITCH_EVENT_SEND_INFO,
 	SWITCH_EVENT_RECV_INFO,
 	SWITCH_EVENT_RECV_RTCP_MESSAGE,
+	SWITCH_EVENT_SEND_RTCP_MESSAGE,
 	SWITCH_EVENT_CALL_SECURE,
 	SWITCH_EVENT_NAT,
 	SWITCH_EVENT_RECORD_START,
@@ -2212,30 +2231,35 @@ typedef enum {
 	SWITCH_CAUSE_INTERWORKING = 127,
 	SWITCH_CAUSE_SUCCESS = 142,
 	SWITCH_CAUSE_ORIGINATOR_CANCEL = 487,
-	SWITCH_CAUSE_CRASH = 500,
-	SWITCH_CAUSE_SYSTEM_SHUTDOWN = 501,
-	SWITCH_CAUSE_LOSE_RACE = 502,
-	SWITCH_CAUSE_MANAGER_REQUEST = 503,
-	SWITCH_CAUSE_BLIND_TRANSFER = 600,
-	SWITCH_CAUSE_ATTENDED_TRANSFER = 601,
-	SWITCH_CAUSE_ALLOTTED_TIMEOUT = 602,
-	SWITCH_CAUSE_USER_CHALLENGE = 603,
-	SWITCH_CAUSE_MEDIA_TIMEOUT = 604,
-	SWITCH_CAUSE_PICKED_OFF = 605,
-	SWITCH_CAUSE_USER_NOT_REGISTERED = 606,
-	SWITCH_CAUSE_PROGRESS_TIMEOUT = 607,
-	SWITCH_CAUSE_INVALID_GATEWAY = 608,
-	SWITCH_CAUSE_GATEWAY_DOWN = 609,
-	SWITCH_CAUSE_INVALID_URL = 610,
-	SWITCH_CAUSE_INVALID_PROFILE = 611,
-	SWITCH_CAUSE_NO_PICKUP = 612,
-	SWITCH_CAUSE_SRTP_READ_ERROR = 613,
-	SWITCH_CAUSE_BOWOUT = 614,
-	SWITCH_CAUSE_BUSY_EVERYWHERE = 615,
-	SWITCH_CAUSE_DECLINE = 616,
-	SWITCH_CAUSE_DOES_NOT_EXIST_ANYWHERE = 617,
-	SWITCH_CAUSE_NOT_ACCEPTABLE = 618,
-	SWITCH_CAUSE_UNWANTED = 619
+	SWITCH_CAUSE_CRASH = 700,
+	SWITCH_CAUSE_SYSTEM_SHUTDOWN = 701,
+	SWITCH_CAUSE_LOSE_RACE = 702,
+	SWITCH_CAUSE_MANAGER_REQUEST = 703,
+	SWITCH_CAUSE_BLIND_TRANSFER = 800,
+	SWITCH_CAUSE_ATTENDED_TRANSFER = 801,
+	SWITCH_CAUSE_ALLOTTED_TIMEOUT = 802,
+	SWITCH_CAUSE_USER_CHALLENGE = 803,
+	SWITCH_CAUSE_MEDIA_TIMEOUT = 804,
+	SWITCH_CAUSE_PICKED_OFF = 805,
+	SWITCH_CAUSE_USER_NOT_REGISTERED = 806,
+	SWITCH_CAUSE_PROGRESS_TIMEOUT = 807,
+	SWITCH_CAUSE_INVALID_GATEWAY = 808,
+	SWITCH_CAUSE_GATEWAY_DOWN = 809,
+	SWITCH_CAUSE_INVALID_URL = 810,
+	SWITCH_CAUSE_INVALID_PROFILE = 811,
+	SWITCH_CAUSE_NO_PICKUP = 812,
+	SWITCH_CAUSE_SRTP_READ_ERROR = 813,
+	SWITCH_CAUSE_BOWOUT = 814,
+	SWITCH_CAUSE_BUSY_EVERYWHERE = 815,
+	SWITCH_CAUSE_DECLINE = 816,
+	SWITCH_CAUSE_DOES_NOT_EXIST_ANYWHERE = 817,
+	SWITCH_CAUSE_NOT_ACCEPTABLE = 818,
+	SWITCH_CAUSE_UNWANTED = 819,
+	SWITCH_CAUSE_NO_IDENTITY = 820,
+	SWITCH_CAUSE_BAD_IDENTITY_INFO = 821,
+	SWITCH_CAUSE_UNSUPPORTED_CERTIFICATE = 822,
+	SWITCH_CAUSE_INVALID_IDENTITY = 823,
+	SWITCH_CAUSE_STALE_DATE = 824
 } switch_call_cause_t;
 
 typedef enum {
@@ -2281,7 +2305,9 @@ typedef enum {
 	SCSC_SPS_PEAK,
 	SCSC_SPS_PEAK_FIVEMIN,
 	SCSC_SESSIONS_PEAK,
-	SCSC_SESSIONS_PEAK_FIVEMIN
+	SCSC_SESSIONS_PEAK_FIVEMIN,
+	SCSC_MDNS_RESOLVE,
+	SCSC_SHUTDOWN_CAUSE
 } switch_session_ctl_t;
 
 typedef enum {
@@ -2297,7 +2323,8 @@ typedef int switch_os_socket_t;
 #define SWITCH_SOCK_INVALID -1
 #endif
 
-typedef struct apr_pool_t switch_memory_pool_t;
+typedef struct fspr_pool_t switch_memory_pool_t;
+typedef void* switch_plc_state_t;
 typedef uint16_t switch_port_t;
 typedef uint8_t switch_payload_t;
 typedef struct switch_app_log switch_app_log_t;
@@ -2314,7 +2341,6 @@ typedef struct switch_rtcp_frame switch_rtcp_frame_t;
 typedef struct switch_channel switch_channel_t;
 typedef struct switch_sql_queue_manager switch_sql_queue_manager_t;
 typedef struct switch_file_handle switch_file_handle_t;
-typedef struct switch_core_session switch_core_session_t;
 typedef struct switch_caller_profile switch_caller_profile_t;
 typedef struct switch_caller_extension switch_caller_extension_t;
 typedef struct switch_caller_application switch_caller_application_t;
@@ -2797,6 +2823,7 @@ typedef struct switch_mm_s {
 	int cbr;
 	float fps;
 	float source_fps;
+	int source_kps;
 	int vbuf;
 	switch_video_profile_t vprofile;
 	switch_video_encode_speed_t vencspd;
